import socket
import sys
import winreg as reg

COMPUTER_NAME = socket.gethostname()


def got_key() -> bool:
    """
    Check if current user has the registry key subsearch for the context menu

    Returns:
        bool: True if key exists, False if key does not exist
    """
    sub_key = r"Software\Classes\*\shell\0.SubSearch\command"
    try:
        with reg.ConnectRegistry(None, reg.HKEY_CURRENT_USER) as hkey:
            reg.OpenKey(hkey, sub_key)
            return True
    except Exception:
        return False


def check_is_exe() -> bool:
    """
    Check if the current user is running subsearch from the executable

    Returns:
        bool: True if subsearch is running as executable, False if subsearch is not running as executable
    """

    if sys.argv[0].endswith("SubSearch.exe"):
        return True
    return False



import sys

from gui import widget_settings
from utils import raw_registry


def main() -> None:
    r"""
    Usages: subsearch [OPTIONS]

    Options:
        --settings                                  Open the GUI settings menu

        --registry-key [add, del]                   Edit the registry
                                                    add: adds the context menu  / replaces the context menu with default values
                                                    del: deletes the context menu
                                                    e.g: subsearch --registry-key add

        --help                                      Prints usage information
    """
    if sys.argv[-1].endswith("subsearch"):
        print(main.__doc__)
        return
    else:
        for num, arg in enumerate(sys.argv[1:], 1):
            if arg.startswith("--settings"):
                widget_settings.show_widget()
                break
            elif arg.startswith("--registry-key") or arg.startswith("--add-key"):
                if sys.argv[num + 1] == "add":
                    raw_registry.add_context_menu()
                    break
                elif sys.argv[num + 1] == "del":
                    raw_registry.remove_context_menu()
                    break
            elif arg.startswith("--help"):
                print(main.__doc__)
                break
            elif len(sys.argv[1:]) == num:
                print("Invalid argument")
                print(main.__doc__)



import requests
from bs4 import BeautifulSoup
from utils import log


# search for file hash
def search_for_hash(url: str, lang: str, hi: str) -> list | None:
    download_url: list = []
    source = requests.get(url)
    scontent = source.content
    doc = BeautifulSoup(scontent, "lxml")
    doc_results = doc.find("table", id="search_results")
    if doc_results is None:
        return None
    tr_name = doc_results.find_all("tr", id=lambda value: value and value.startswith("name"))
    for item in tr_name:
        tl = [a["title"] for a in item.find_all("a", title=lang)]
        if lang in tl:
            hi = item.find("img", alt="Subtitles for hearing impaired")
            if hi is not None and hi is False:
                log.output(f"Found HI-subtitle but skipping, 'cus hearing impaired is set to '{hi}'")
                continue
            if hi is None and hi is True:
                log.output(f"Found nonHI-subtitle but skipping, 'cus hearing impaired is set to '{hi}'")
                continue

            title_name = item.find("a", class_="bnone").text.replace("\n", "").replace("\t", "").replace("(", " (")
            log.output(f"{title_name} matched file hash")
            th = [
                a["href"]
                for a in item.find_all(
                    "a",
                    href=lambda value: value and value.startswith("/en/subtitleserve/sub/"),
                )
            ]
            th = th[0] if th is not None else None
            link = f"https://www.opensubtitles.org{th}"
            download_url.append(link) if th is not None else None
    if len(download_url) == 0:
        return None
    return download_url



from data import __video_directory__
from utils import log

from . import opensubtitles_soup


# decides what to do with all the scrape data
def scrape(param, lang: str, hi: str) -> list | None:
    """
    Scrape opensubtitles for subtitles using the given parameters.

    Args:
        param (_type_): url_opensubtitles, file_hash
        lang (str): desired language
        hi (str): if hearing impaired subtitles are desired

    Returns:
        list | None: download links for the subtitle
    """
    to_be_downloaded = opensubtitles_soup.search_for_hash(param.url_opensubtitles, lang, hi)
    if to_be_downloaded is None:
        if param.tv_series:
            log.output(f"No TV-series found matching hash {param.file_hash}")
        else:
            log.output(f"No movies found matching hash {param.file_hash}")
        return None

    download_info: list = []
    log.output(f"Preparing  hash {param.file_hash} for download")
    for current_num, (dl_url) in enumerate(to_be_downloaded):
        total_num = len(to_be_downloaded)
        current_num += 1
        file_path = f"{__video_directory__}\\opensubtitles_{current_num}.zip"
        current_num = (file_path, dl_url, current_num, total_num)
        download_info.append(current_num)
    log.output(f"Done with tasks")
    return download_info



import json
import os
import tkinter as tk
import webbrowser

from data import __data__, __version__
from utils import current_user, raw_config, raw_registry, updates

from . import tkinter_data as tkd
from . import tools, widget_root

LANGUAGES = raw_config.get("languages")
OTH_LANGUAGES = raw_config.get("other_languages")
LANGUAGE, LANG_ABBR = raw_config.get("language")
HEARING_IMPARED = raw_config.get("hearing_impaired")
PCT = raw_config.get("percentage")
SHOW_TERMINAL = raw_config.get("show_terminal")
CM_ICON = raw_config.get("cm_icon")
DL_WINDOW = raw_config.get("show_download_window")
AVAIL_EXT = raw_config.get("file_ext")

# set which language of the subtitles  should be included in the search
class SelectLanguage(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{LANGUAGE}, {LANG_ABBR}")
        number_of_buttons = len(LANGUAGES)
        self.rowcount = 0
        self.colcount = 1
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Selected language", sticky="w", font=tkd.Font.cas8b)
        self.clabel = tools.Create.label(
            self,
            textvar=self.string_var,
            fg=tkd.Color.yellow,
            col=2,
            font=tkd.Font.cas8b,
        )
        for i in range(number_of_buttons):
            self.rowcount += 1
            if self.rowcount == 8:
                self.colcount += 1
                self.rowcount = 1
            tools.Create.button(
                self,
                text=LANGUAGES[i],
                row=self.rowcount + 1,
                col=self.colcount,
                height=2,
                width=24,
                padx=2,
                bind_to=self.set_language,
            )
        tools.Create.button(
            self,
            abgc=tkd.Color.light_black,
            bge=tkd.Color.light_black,
            fg=tkd.Color.light_black,
            fge=tkd.Color.light_black,
            row=self.rowcount + 2,
            col=self.colcount,
            height=2,
            width=24,
        )
        self.entry = tk.Entry(self, text="", width=28, bd=0, font=tkd.Font.cas8b, justify="center")
        self.entry.insert(0, "🞂 Enter language here 🞀")
        self.entry.configure(
            bg=tkd.Color.light_black,
            fg=tkd.Color.purple,
            insertbackground=tkd.Color.purple,
        )
        self.entry.grid(ipady=8, padx=2, pady=2, row=self.rowcount + 2, column=self.colcount)
        self.add_button = tools.Create.button(
            self,
            text="Add",
            abgc=tkd.Color.purple,
            bge=tkd.Color.black,
            fg=tkd.Color.white_grey,
            fge=tkd.Color.purple,
            row=self.rowcount + 3,
            col=self.colcount,
            height=2,
            width=10,
            padx=5,
            sticky="w",
            bind_to=self.add_language,
        )

        self.see_other_langs = tools.Create.button(
            self,
            text="∙ ∙ ∙",
            abgc=tkd.Color.purple,
            bge=tkd.Color.black,
            fg=tkd.Color.white_grey,
            fge=tkd.Color.purple,
            row=self.rowcount + 3,
            col=self.colcount,
            height=2,
            width=10,
            padx=5,
            sticky="e",
        )
        self.entry.bind("<Enter>", self.entry_enter)
        self.see_other_langs.bind("<Enter>", self.popup_window)
        self.entry.bind("<Return>", self.add_language)
        self.configure(bg=tkd.Color.dark_grey)

    # pop up window with list of other languages
    def popup_window(self, event):
        # * see 'cleaner' version of this function src.gui.tools.Tooltip
        self.clear_entry()
        rows = len(OTH_LANGUAGES) / 2
        cols = 2
        row_size_y = 20.16
        col_size_x = 113
        csx = round(cols * col_size_x)
        csy = round(rows * row_size_y)

        self.toplvl = tk.Toplevel(background=tkd.Color.light_black, borderwidth=0)
        self.toplvl.overrideredirect(True)

        self.frame = tk.Frame(
            self.toplvl,
            background=tkd.Color.dark_grey,
            width=csx,
            height=csy,
            borderwidth=0,
        )
        self.frame.place(relx=0.5, rely=0.5, anchor="center")

        root_x = root.winfo_rootx() + tkd.Window.width + 10
        root_y = root.winfo_rooty() + 37
        self.toplvl.geometry(f"{csx}x{csy}+{root_x}+{root_y}")
        for num, i in zip(range(0, 50), OTH_LANGUAGES):
            tools.Create.label(
                self.frame,
                bg=tkd.Color.dark_grey,
                text=i,
                font=tkd.Font.cas8,
                row=num if num < 25 else num - 25,
                col=0 if num < 25 else 1,
                sticky="w",
                padx=0,
                pady=0,
            )
        self.see_other_langs.configure(fg=tkd.Color.purple)
        self.see_other_langs.bind("<Leave>", self.popup_window_destroy)

    def popup_window_destroy(self, event):
        self.fill_entry()
        self.see_other_langs.configure(fg=tkd.Color.white_grey)
        self.see_other_langs.bind("<Enter>", self.popup_window)
        self.toplvl.destroy()

    # entry functions
    def entry_enter(self, event):
        if self.entry.get() == "🞂 Enter language here 🞀" or self.entry.get() == "E.g: Czech, cs":
            self.clear_entry()
            self.entry.bind("<Leave>", self.entry_leave)

    def entry_leave(self, event):
        if self.entry.get() == "" or self.entry.get() == "E.g: Czech, cs":
            self.fill_entry()
            self.entry.bind("<Enter>", self.entry_enter)

    def clear_entry(self):
        self.entry.delete(0, "end")
        self.entry.insert(0, "")
        self.entry.configure(fg=tkd.Color.purple)

    def fill_entry(self):
        self.entry.delete(0, "end")
        self.entry.insert(0, "🞂 Enter language here 🞀")
        self.entry.configure(fg=tkd.Color.purple)

    # set language
    def set_language(self, event):
        btn = event.widget
        self.string_var.set(btn.cget("text"))
        update_svar = self.string_var.get()
        raw_config.set_json("language", update_svar)

    # add language from entry
    def add_language(self, event):
        x = self.entry.get()
        for i in OTH_LANGUAGES:
            if x == i:
                self.string_var.set(self.entry.get())
                self.entry.configure(fg=tkd.Color.white_grey)
                update_svar = self.string_var.get()
                raw_config.set_json("language", update_svar)
                return
        self.entry.delete(0, "end")
        self.entry.insert(0, "E.g: Czech, cs")
        self.entry.configure(fg=tkd.Color.red)


# set HI, none-HI or both HI and none-HI subtitles should be included in the search
class HearingImparedSubs(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{HEARING_IMPARED}")
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(
            self,
            text="Hearing impaired subtitles",
            sticky="w",
            font=tkd.Font.cas8b,
            anchor="w",
        )
        self.clabel = tools.Create.label(self, textvar=self.string_var, fg=tkd.Color.blue, col=2, font=tkd.Font.cas8b)
        tools.Create.button(
            self,
            text="True",
            width=7,
            sticky="e",
            bind_to=self.button_set_true,
            tip_show=True,
            tip_text="Only use hearing impaired subtitles",
        )
        tools.Create.button(
            self,
            text="False",
            width=7,
            sticky="w",
            bind_to=self.button_set_false,
            tip_show=True,
            tip_text="Only use non-hearing impaired subtitles",
        )
        tools.Create.button(
            self,
            text="Both",
            width=7,
            bind_to=self.button_set_both,
            tip_show=True,
            tip_text="Use both hearing impaired and regular subtitles",
        )

        tools.ColorPicker(self.string_var, self.clabel)
        self.configure(bg=tkd.Color.dark_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("hearing_impaired", True)

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("hearing_impaired", False)

    def button_set_both(self, event):
        self.string_var.set(f"Both")
        tools.ColorPicker(self.string_var, self.clabel)
        update_svar = self.string_var.get().split(" ")[0]
        raw_config.set_json("hearing_impaired", update_svar)


# set how closely the subtitle name should match the release name of the media file
class SearchThreshold(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{PCT} %")
        self.pct = PCT
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Search threshold", sticky="w", font=tkd.Font.cas8b)
        self.clabel = tools.Create.label(self, textvar=self.string_var, fg=tkd.Color.blue, col=2, font=tkd.Font.cas8b)
        tools.Create.button(
            self,
            text="+",
            sticky="e",
            bind_to=self.button_add_5,
            tip_show=True,
            tip_text="Add 5% to the search threshold\nA higher value means less chance of finding subtitles that are not synced witht the movie/series",
        )
        tools.Create.button(
            self,
            text="-",
            sticky="w",
            bind_to=self.button_sub_5,
            tip_show=True,
            tip_text="Subtract 5% from the search threshold\nA lower value means more subtitles will be found and downloaded",
        )
        tools.ColorPicker(self.string_var, self.clabel, self.pct)
        self.configure(bg=tkd.Color.dark_grey)

    def button_add_5(self, event):
        self.pct += 5 if self.pct < 100 else 0
        self.string_var.set(f"{self.pct} %")

        tools.ColorPicker(self.string_var, self.clabel, self.pct)
        update_svar = int(self.pct)
        raw_config.set_json("percentage_pass", update_svar)

    def button_sub_5(self, event):
        self.pct -= 5 if self.pct > 0 else 0
        self.string_var.set(f"{self.pct} %")
        tools.ColorPicker(self.string_var, self.clabel, self.pct)
        update_svar = int(self.pct)
        raw_config.set_json("percentage_pass", update_svar)


# remove or restore the context menu option when right-clicking
class ShowContextMenu(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"True")
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Show context menu", sticky="w", font=tkd.Font.cas8b)
        self.clabel = tools.Create.label(
            self,
            textvar=self.string_var,
            fg=tkd.Color.blue,
            col=2,
            font=tkd.Font.cas8b,
            anchor="center",
        )
        tools.Create.button(
            self,
            text="True",
            sticky="e",
            bind_to=self.button_set_true,
            tip_show=True,
            tip_text="Add SubSearch to the context menu when you right click inside a folder",
        )
        tools.Create.button(
            self,
            text="False",
            sticky="w",
            bind_to=self.button_set_false,
            tip_show=True,
            tip_text="Remove SubSearch from the context menu\nUsed to 'uninstall' SubSearch",
        )
        tools.ColorPicker(self.string_var, self.clabel)
        self.configure(bg=tkd.Color.dark_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tools.ColorPicker(self.string_var, self.clabel)
        from utils import raw_registry

        raw_registry.add_context_menu()
        raw_registry.write_all_valuex()

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tools.ColorPicker(self.string_var, self.clabel)
        from utils import raw_registry

        raw_registry.remove_context_menu()


# * need cleanup and comments but works
# remove or restore the context menu option when right-clicking
class AssociateExtensions(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)

        self.ext_window_show = False
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Associated extensions", sticky="w", font=tkd.Font.cas8b)

        self.ext_button = tools.Create.button(
            self,
            text="Show menu",
            height=1,
            width=24,
            tip_show=True,
            tip_text="Which file extension to show the context menu on",
        )
        self.fesm = FileExtSubMenu(self)
        self.ext_button.bind("<Button-1>", self.toggle_window)
        self.configure(bg=tkd.Color.dark_grey)

    def toggle_window(self, event):
        self.ext_window_show = self.fesm.window_showing_check()
        if self.ext_window_show:
            self.fesm.toggle_window()
            self.ext_window_show = self.fesm.window_showing_check()
        else:
            self.fesm.show()
            self.ext_window_show = self.fesm.window_showing_check()


class FileExtSubMenu(tk.Toplevel):
    def __init__(self, parent):
        self.parent = parent
        self.window_showing = False

    def show(self):
        self.files = []
        self.btn = []
        self.window_showing = True
        tk.Toplevel.__init__(self, self.parent)
        self.configure(background=tkd.Color.light_black)
        # remove the standard window titlebar from the tooltip
        self.overrideredirect(True)
        self.attributes("-topmost", True)
        button_width = 10
        exit_size = 19
        border_size = 2 * 2
        top_bar = tk.Frame(self, background=tkd.Color.light_black)
        _frame = tk.Frame(self, background=tkd.Color.light_grey)
        data = raw_config.get_json()
        for i in range(len(AVAIL_EXT.keys())):
            self.files.append("Button" + str(i))

        for i, ext in zip(range(len(self.files)), AVAIL_EXT.keys()):
            rownum = i
            if (rownum % 2) == 0:
                colnum = 0
            else:
                rownum -= 1
                colnum = 1

            self.btn.append(
                tk.Button(
                    _frame,
                    text=ext,
                    font=tkd.Font.cas8b,
                    bg=tkd.Color.dark_grey,
                    fg=tkd.Color.white_grey,
                    activebackground=tkd.Color.dark_grey,
                    activeforeground=tkd.Color.yellow,
                    width=button_width,
                    bd=0,
                    command=lambda c=i: self.btn[c].cget("text"),
                )
            )
            if data["file_ext"][ext] is False:
                self.btn[i].configure(fg=tkd.Color.red)
            elif data["file_ext"][ext] is True:
                self.btn[i].configure(fg=tkd.Color.green)

            self.btn[i].grid(row=rownum, column=colnum)
            self.btn[i].bind("<Button-1>", self.set_language)
        # get size of the label to use later for positioning and sizing of the tooltip, + 2 to account padx/pady 1px
        _x, _y = self.btn[0].winfo_reqwidth() + 2, self.btn[0].winfo_reqheight() + 2
        # 2 rows half is length, 2 columns is width
        nrow = round(len(AVAIL_EXT) / 2)
        ncol = round(2)
        x = round(_x * ncol)
        y = round(_y * nrow)
        x_offset = tkd.Window.width + button_width - _x
        y_offset = tkd.Window.height - y - nrow
        # set the size of the tooltip background to be 1px larger than the label
        _frame.configure(width=x + border_size, height=y + border_size)
        top_bar.configure(width=x + border_size, height=exit_size)
        # offset the frame 1px from edge of the tooltip corner
        _frame.place(x=2, y=exit_size)
        top_bar.place(x=x, y=0, anchor="ne")
        # top_bar.lift()
        root_x = self.parent.winfo_rootx() + x_offset  # offset tooltip by extra 4px so it doesn't overlap the parent
        root_y = root.winfo_rooty() + y_offset  # place ext window at the bot of the roo offset by ext window hight
        # set position of the tooltip, size and add 2px around the tooltip for a 1px border
        self.geometry(f"{x}x{y+nrow}+{root_x}+{root_y}")

        self.exit_path = tools.buttons("exit.png")
        self.exit_grey_path = tools.buttons("exit_grey.png")
        self.exit_png = tk.PhotoImage(file=self.exit_path)
        self.exit_grey_png = tk.PhotoImage(file=self.exit_grey_path)
        self.exit = tk.Canvas(
            self,
            width=exit_size,
            height=exit_size,
            bg=tkd.Color.light_black,
            highlightthickness=0,
        )
        self.exit.place(relx=1, y=0, anchor="ne")
        self.update_img(self.exit, self.exit_grey_png)
        self.exit.bind("<Enter>", self.exit_enter)
        self.exit.bind("<Leave>", self.exit_leave)
        top_bar.bind("<Button-1>", self.tb_press)
        top_bar.bind("<B1-Motion>", self.tb_drag)
        return self.window_showing

    def set_language(self, event):

        btn = event.widget
        data = raw_config.get_json()
        if data["file_ext"][btn.cget("text")] is True:
            data["file_ext"][btn.cget("text")] = False
            btn.configure(fg=tkd.Color.red)
        else:
            data["file_ext"][btn.cget("text")] = True
            btn.configure(fg=tkd.Color.green)
        file = os.path.join(__data__, "config.json")
        with open(file, "w") as f:
            f.seek(0)
            json.dump(data, f, indent=4)
            f.truncate()

    def exit_release(self, event):
        from utils import raw_registry

        raw_registry.write_all_valuex()
        self.window_showing = False
        self.destroy()

    def toggle_window(self):
        from utils import raw_registry

        self.destroy()
        self.window_showing = False
        raw_registry.write_all_valuex()

    def window_showing_check(self):
        return self.window_showing

    def exit_press(self, event):
        self.exit.configure(bg=tkd.Color.dark_red)
        self.exit.bind("<ButtonRelease-1>", self.exit_release)

    def exit_enter(self, event):
        self.exit.configure(bg=tkd.Color.red)
        self.update_img(self.exit, self.exit_png)
        self.exit.bind("<ButtonPress-1>", self.exit_press)

    def exit_leave(self, event):
        self.exit.configure(bg=tkd.Color.light_black)
        self.update_img(self.exit, self.exit_grey_png)
        self.exit.unbind("<ButtonRelease-1>")

    def update_img(self, canvas, img):
        canvas.delete("all")
        canvas.create_image(9, 9, image=img)
        canvas.photoimage = img

    def tb_press(self, event):
        self._offsetx = self.winfo_pointerx() - self.winfo_rootx()
        self._offsety = self.winfo_pointery() - self.winfo_rooty()

    def tb_drag(self, event):
        x = self.winfo_pointerx() - self._offsetx
        y = self.winfo_pointery() - self._offsety
        self.geometry(f"+{x}+{y}")


# remove or restore the icon next to the context menu option when right clicking
class ShowContextMenuIcon(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{CM_ICON}")
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Show context menu icon", sticky="w", font=tkd.Font.cas8b)
        self.clabel = tools.Create.label(
            self,
            textvar=self.string_var,
            fg=tkd.Color.blue,
            col=2,
            font=tkd.Font.cas8b,
        )
        tools.Create.button(
            self,
            text="True",
            sticky="e",
            bind_to=self.button_set_true,
            tip_show=True,
            tip_text="Add a icon next to SubSearch in the context menu",
        )
        tools.Create.button(
            self,
            text="False",
            sticky="w",
            bind_to=self.button_set_false,
            tip_show=True,
            tip_text="Remove the icon next to SubSearch in the context menu",
        )
        tools.ColorPicker(self.string_var, self.clabel)
        self.configure(bg=tkd.Color.dark_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("context_menu_icon", True)
        from utils import raw_registry

        raw_registry.write_valuex("icon")

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("context_menu_icon", False)
        from utils import raw_registry

        raw_registry.write_valuex("icon")


# remove or restore the icon next to the context menu option when right clicking
class ShowDownloadWindow(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"{DL_WINDOW}")
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Show download window", sticky="w", font=tkd.Font.cas8b)
        self.clabel = tools.Create.label(
            self,
            textvar=self.string_var,
            fg=tkd.Color.blue,
            col=2,
            font=tkd.Font.cas8b,
        )
        tools.Create.button(
            self,
            text="True",
            sticky="e",
            bind_to=self.button_set_true,
            tip_show=True,
            tip_text="If no subtitles are found show a window with the disregarded subtitles with download buttons to each of them",
        )
        tools.Create.button(
            self,
            text="False",
            sticky="w",
            bind_to=self.button_set_false,
            tip_show=True,
            tip_text="No window will be shown if no subtitles are found\nThe list can be found in search.log",
        )
        tools.ColorPicker(self.string_var, self.clabel)
        self.configure(bg=tkd.Color.dark_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("show_download_window", True)

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("show_download_window", False)


# show a terminal with what the code is doing while searching
class ShowTerminalOnSearch(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()

        self.string_var.set(f"{SHOW_TERMINAL}")
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(self, text="Show terminal on search", sticky="w", font=tkd.Font.cas8b)
        self.clabel = tools.Create.label(
            self,
            textvar=self.string_var,
            fg=tkd.Color.blue,
            col=2,
            font=tkd.Font.cas8b,
        )
        if current_user.check_is_exe() is False:
            tools.Create.button(
                self,
                text="True",
                sticky="e",
                bind_to=self.button_set_true,
                tip_show=True,
                tip_text="Show the terminal when searching for subtitles\nEverything shown in the terminal is avalible in search.log",
            )
            tools.Create.button(
                self,
                text="False",
                sticky="w",
                bind_to=self.button_set_false,
                tip_show=True,
                tip_text="Hide the terminal when searching for subtitles",
            )
        tools.ColorPicker(self.string_var, self.clabel)
        self.configure(bg=tkd.Color.dark_grey)

    def button_set_true(self, event):
        self.string_var.set(f"True")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("show_terminal", True)
        raw_registry.write_valuex("command")

    def button_set_false(self, event):
        self.string_var.set(f"False")
        tools.ColorPicker(self.string_var, self.clabel)
        raw_config.set_json("show_terminal", False)
        raw_registry.write_valuex("command")


# check for new updates on the github repository
class CheckForUpdates(tk.Frame):
    def __init__(self, parent):
        tk.Frame.__init__(self, parent)
        self.string_var = tk.StringVar()
        self.string_var.set(f"")
        for i in range(1, 4):
            tools.Create.label(self, text=tkd.Misc.col58, col=i, font=tkd.Font.cas8)
        tools.Create.label(
            self,
            text=f"SubScene version {__version__}",
            sticky="w",
            font=tkd.Font.cas8b,
        )
        tools.Create.label(self, textvar=self.string_var, fg=tkd.Color.blue, col=2, font=tkd.Font.cas8b)
        tools.Create.button(
            self,
            text="Check for updates",
            height=2,
            width=24,
            fge=tkd.Color.green,
            bind_to=self.button_check,
        )
        self.configure(bg=tkd.Color.dark_grey)

    def button_check(self, event):
        self.string_var.set(f"Searching for updates...")
        value, release_type = updates.is_new_version_available()
        latest_version = updates.check_for_updates()
        if value:
            self.string_var.set(f"New version available!")
            tools.Create.button(
                self,
                text=f"Get {latest_version}",
                height=2,
                width=24,
                bind_to=self.button_download,
            )

        if value is False and release_type is None:
            self.string_var.set(f"You are up to date!")
        elif value is False and release_type is None:
            self.string_var.set(f"New {release_type} update available")
        elif value is False and release_type is not None:
            self.string_var.set(f"Branch ahead of main branch")

    def button_download(self, event):
        webbrowser.open("https://github.com/vagabondHustler/SubSearch/releases")


def show_widget():
    global root
    if current_user.got_key() is False:
        raw_config.set_default_json()
        raw_registry.add_context_menu()

    root = widget_root.main()
    SelectLanguage(root).pack(anchor="center")
    tk.Frame(root, bg=tkd.Color.dark_grey).pack(anchor="center", expand=True)
    HearingImparedSubs(root).pack(anchor="center")
    SearchThreshold(root).pack(anchor="center")
    tk.Frame(root, bg=tkd.Color.dark_grey).pack(anchor="center", expand=True)
    ShowContextMenu(root).pack(anchor="center")
    ShowContextMenuIcon(root).pack(anchor="center")
    AssociateExtensions(root).pack(anchor="center")
    ShowDownloadWindow(root).pack(anchor="center")
    if current_user.check_is_exe() is False:
        ShowTerminalOnSearch(root).pack(anchor="center")
    tk.Frame(root, bg=tkd.Color.dark_grey).pack(anchor="center", expand=True)
    CheckForUpdates(root).pack(anchor="center")
    tk.Frame(root, bg=tkd.Color.dark_grey).pack(anchor="center", expand=True)

    root.mainloop()



import ctypes
import os
import time

from data import __version__, __video_directory__, __video_name__, __video_path__
from gui import widget_download, widget_settings
from scraper import opensubtitles, subscene
from utils import current_user, file_manager, file_parser, log, raw_config, raw_registry


class SubSearch:
    def __init__(self) -> None:
        self.start = time.perf_counter()
        ctypes.windll.kernel32.SetConsoleTitleW(f"SubSearch - {__version__}")
        if current_user.got_key() is False:
            raw_config.set_default_json()
            raw_registry.add_context_menu()
        self.show_terminal = False if current_user.check_is_exe() else raw_config.get("show_terminal")
        self.file_exist = True if __video_name__ is not None else False
        self.lang, self.lang_abbr = raw_config.get("language")
        self.hi = raw_config.get("hearing_impaired")
        self.pct = raw_config.get("percentage")
        self.show_dl_window = raw_config.get("show_download_window")
        if self.file_exist:
            self.file_hash = file_manager.get_hash(__video_path__)
            self.param = file_parser.get_parameters(__video_name__, self.file_hash, self.lang_abbr)
            log.parameters(self.param, self.lang, self.lang_abbr, self.hi, self.pct)
        if self.file_exist is False:
            widget_settings.show_widget()

    def opensubtitles_scrape(self):
        if self.file_exist:
            log.output("")
            log.output("[Searching on opensubtitles]")
            self.scrape_opensubtitles = opensubtitles.scrape(self.param, self.lang, self.hi)

    def subscene_scrape(self):
        if self.file_exist:
            log.output("")
            log.output("[Searching on subscene]")
            self.scrape_subscene = subscene.scrape(
                self.param, self.lang, self.lang_abbr, self.hi, self.pct, self.show_dl_window
            )

    def procsess_files(self):
        if self.file_exist:
            if self.scrape_opensubtitles is not None:
                log.output("")
                log.output("[Downloading from Opensubtitles]")
                for item in self.scrape_opensubtitles:
                    file_manager.download_zip(item)

            if self.scrape_subscene[0] is not None:
                log.output("")
                log.output("[Downloading from Subscene]")
                for item in self.scrape_subscene:
                    file_manager.download_zip(item)

            if self.scrape_opensubtitles is None and self.scrape_subscene is None:
                dl_data = f"{__video_directory__}\\__subsearch__dl_data.tmp"
                if self.show_dl_window is True and os.path.exists(dl_data):
                    widget_download.show_widget()
                    file_manager.clean_up(__video_directory__, dl_data)

            log.output("")
            log.output("[Procsessing files]")
            file_manager.extract_zips(__video_directory__, ".zip")
            file_manager.clean_up(__video_directory__, ".zip")
            file_manager.clean_up(__video_directory__, ").nfo")
            file_manager.rename_best_match(f"{self.param.release}.srt", __video_directory__, ".srt")

    def _exit_(self):
        elapsed = time.perf_counter() - self.start
        log.output("")
        log.output(f"Finished in {elapsed} seconds")
        try:
            while self.show_terminal and current_user.check_is_exe() is False:
                print("\nCtrl + c to exit")
                time.sleep(900)
        except KeyboardInterrupt:
            pass


def main():
    ss = SubSearch()
    ss.opensubtitles_scrape()
    ss.subscene_scrape()
    ss.procsess_files()
    ss._exit_()


if __name__ == "__main__":
    main()



from dataclasses import dataclass


@dataclass
class Window:
    """
    Class for storing settings for window size.

    Args:
    width: width of window in pixels.
    height: height of window in pixels.
    """

    width: int = 700
    height: int = 700


@dataclass
class Color:
    """
    Class for storing colors for widgets.

    Args:
    purple: "#b294bb"
    red: "#bc473b"
    dark_red: "#89332a"
    red_brown: "#b26947"
    orange: "#ab7149"
    light_orange: "#de935f"
    yellow: "#f0c674"
    blue: "#81a2be"
    cyan: "#82b3ac"
    green: "#9fa65d"
    green_brown: "#a59256"
    grey: "#4c4c4c"
    light_grey: "#727272"
    dark_grey: "#1A1A1A"
    silver_grey: "#8a8a8a"
    white_grey: "#bdbdbd"
    light_black: "#0e0e0e"
    black: "#151515"
    dark_black: "#000000"
    """

    purple: str = "#b294bb"
    red: str = "#bc473b"
    dark_red: str = "#89332a"
    red_brown: str = "#b26947"
    orange: str = "#ab7149"
    light_orange: str = "#de935f"
    yellow: str = "#f0c674"
    blue: str = "#81a2be"
    cyan: str = "#82b3ac"
    green: str = "#9fa65d"
    green_brown: str = "#a59256"
    grey: str = "#4c4c4c"
    light_grey: str = "#727272"
    dark_grey: str = "#1A1A1A"
    silver_grey: str = "#8a8a8a"
    white_grey: str = "#bdbdbd"
    light_black: str = "#0e0e0e"
    black: str = "#151515"
    dark_black: str = "#000000"


@dataclass
class Font:
    """
    Class for storing fonts for widgets

    Args:
    cas6b: "Cascadia 6 bold"
    cas8: "Cascadia 8"
    cas8i: "Cascadia 8 italic"
    cas8b: "Cascadia 8 bold"
    cas10b: "Cascadia 10 bold"
    cas11: "Cascadia 11"
    cas20b: "Cascadia 20 bold"
    """

    cas6b: str = "Cascadia 6 bold"
    cas8: str = "Cascadia 8"
    cas8i: str = "Cascadia 8 italic"
    cas8b: str = "Cascadia 8 bold"
    cas10b: str = "Cascadia 10 bold"
    cas11: str = "Cascadia 11"
    cas20b: str = "Cascadia 20 bold"


@dataclass
class Misc:
    """
    Class for storing misc settings.

    Args:
    col58: " " * 58
    """

    col58: str = " " * 58



from data import __video_directory__
from utils import log, string_parser

from . import subscene_soup


# check if dict is of movies
def is_movie(key: str, param=None) -> bool:
    """
    Check if key is a movie, by checking if the title is followed by a year. Title.Of.The.Movie.YEAR.Source.Codec-GROUP

    Args:
        key (str): name of the file plus release information
        param (SearchParameters, optional): title, year. Defaults to None.

    Returns:
        bool: True if title is followed by a year, False otherwise
    """
    if key.lower() == f"{param.title} ({param.year})":
        log.output(f"Movie {key} found")
        return True
    return False


# check if the movie might have been released the year before
def try_the_year_before(key: str, param=None) -> bool:
    """
    Some releases are released close to the next year. If so, the year might differ from the year in the title.
    This function subtracts one year from the year in the title and checks if the release is in the list.

    Args:
        key (str): name of the file plus release information
        param (SearchParameters, optional): title, year. Defaults to None.

    Returns:
        bool: True if the release is found one year before, False otherwise
    """
    if param.year == "N/A":
        return False
    year = int(param.year) - 1
    the_year_before = f"{param.title} ({year})"
    if key.lower().startswith(the_year_before):
        log.output(f"Movie {key} found")
        return True


# check if dict is of tv-series
def is_tv_series(key: str, lang_abbr: str, param=None) -> bool:
    """
    Check if key is a movie, by checking if the title is followed by a SeasonEpisode. Title.Of.The.S00E00.Source.Codec-GROUP

    Args:
        key (str): name of the file plus release information
        lang_abbr (str): language abbreviation for ordinal numbers
        param (SearchParameters, optional): title, season_ordinal, tv_series. Defaults to None.

    Returns:
        bool: True if title is followed by a SeasonEpisode, False otherwise
    """
    if param.title and param.season_ordinal in key.lower() and param.tv_series and lang_abbr:
        log.output(f"TV-Series {key} found")
        return True
    return False


# check str is above percentage threshold
def is_threshold(key: str, number: int, pct: int, param=None) -> bool:
    """
    Check if the title and release information is equal or above the percentage threshold.

    Args:
        key (str): name of the file plus release information
        number (int): percentage threshold
        pct (int): actual percentage
        param (_type_, optional): title, season, episode, tv_series. Defaults to None.

    Returns:
        bool: True if the percentage is equal or above the threshold, False otherwise
    """
    if number >= pct or param.title and f"{param.season}{param.episode}" in key.lower() and param.tv_series:
        return True
    return False


# log and sort list
def log_and_sort_list(list_of_tuples: list, pct: int) -> list:
    """
    Log all the result and sort the list from highest to lowest percentage.

    Args:
        list_of_tuples (list): name of the file plus release information
        pct (int): matching percentage

    Returns:
        list: sorted list of tuples(release, url)
    """
    list_of_tuples.sort(key=lambda x: x[0], reverse=True)
    log.output("\n[Sorted List from Subscene]")
    hbd_printed = False
    hnbd_printed = False
    for i in list_of_tuples:
        name = i[1]
        url = i[2]
        if i[0] >= pct and not hbd_printed:
            log.output(f"--- Has been downloaded ---\n")
            hbd_printed = True
        if i[0] <= pct and not hnbd_printed:
            log.output(f"--- Has not been downloaded ---\n")
            hnbd_printed = True
        log.output(f"{name}\n{url}\n")
    return list_of_tuples


# decides what to do with all the scrape data
def scrape(param, lang: str, lang_abbr: str, hi: str, pct: int, show_dl_window: str) -> list | None:
    # search for titles
    to_be_scraped: list = []
    title_keys = subscene_soup.search_for_title(param.url_subscene)
    if title_keys == "ERROR: CAPTCHA PROTECTION":
        log.output(f"Captcha protection detected. Please try again later.")
        return None
    for key, value in title_keys.items():
        if is_movie(key, param):
            to_be_scraped.append(value) if value not in (to_be_scraped) else None
        if try_the_year_before(key, param):
            to_be_scraped.append(value) if value not in (to_be_scraped) else None
        if is_tv_series(key, lang_abbr, param):
            to_be_scraped.append(value) if value not in (to_be_scraped) else None
    log.output("Done with task\n") if len(to_be_scraped) > 0 else None

    # exit if no titles found
    if len(to_be_scraped) == 0:
        if param.tv_series:
            log.output("")
            log.output(f"No TV-series found matching {param.title}")
        else:
            log.output("")
            log.output(f"No movies found matching {param.title}")
        return None

    # search title for subtitles
    to_be_downloaded: list = []
    to_be_sorted: list = []
    while len(to_be_scraped) > 0:
        for url in to_be_scraped:
            log.output(f"[Searching for subtitles]")
            sub_keys = subscene_soup.search_title_for_sub(lang, hi, url)
            break
        for key, value in sub_keys.items():
            number = string_parser.pct_value(key, param.release)
            log.output(f"[Found]: {key}")
            lenght_str = sum(1 for char in f"[{number}% match]:")
            formatting_spaces = " " * lenght_str
            _name = f"[{number}% match]: {key}"
            _url = f"{formatting_spaces} {value}"
            to_be_sorted_value = number, _name, _url
            to_be_sorted.append(to_be_sorted_value)
            if is_threshold(key, number, pct, param):
                to_be_downloaded.append(value) if value not in to_be_downloaded else None
        to_be_scraped.pop(0) if len(to_be_scraped) > 0 else None
        sorted_list = log_and_sort_list(to_be_sorted, pct)
        log.output("Done with tasks")

    # exit if no subtitles found
    if len(to_be_downloaded) == 0:
        log.output("")
        log.output(f"No subtitles to download for {param.release}")
        if show_dl_window and len(sorted_list) > 0:
            with open("__subsearch__dl_data.tmp", "w", encoding="utf8") as f:
                for i in range(len(sorted_list)):
                    name, _link = sorted_list[i][1], sorted_list[i][2]
                    link = _link.replace(" ", "")
                    f.writelines(f"{name} {link}")
                    f.write("\n")

            return None
        return None

    download_info: list = []
    for current_num, (dl_url) in enumerate(to_be_downloaded):
        total_num = len(to_be_downloaded)
        current_num += 1
        root_dl_url = subscene_soup.get_download_url(dl_url)
        file_path = f"{__video_directory__}\\__subsearch__subscene_{current_num}.zip"
        current_num = (file_path, root_dl_url, current_num, total_num)
        download_info.append(current_num)
    return download_info



import re

import cloudscraper
from data import __version__

SCRAPER = cloudscraper.create_scraper(browser={"browser": "chrome", "platform": "android", "desktop": False})


def check_for_updates() -> str:
    source = SCRAPER.get("https://raw.githubusercontent.com/vagabondHustler/SubSearch/main/src/subsearch/data/version.json")
    scontent = source.content
    _string = str(scontent)
    _string_no_qoute = _string.replace('"', " ")
    _string_items = _string_no_qoute.split(" ")
    for i in _string_items:
        if i.startswith("v") and i[-1].isnumeric():
            latest_version = i
            return latest_version


def is_new_version_available() -> tuple[bool, str]:
    """
    Compare local version with latest version on github

    Returns:
        tuple[bool, str]: True, None if local version is less than repo, False, "newer" if local version is greater than repo, False, else False None
    """

    local_version = re.findall(r"\d+", __version__)
    repo_version = re.findall(r"\d+", check_for_updates())

    if local_version < repo_version:
        return True, None
    if local_version > repo_version:
        return False, "newer"
    return False, None



import logging
import os
from datetime import datetime

from data import __version__, __video_directory__

CWD = __video_directory__
if __video_directory__ is None:
    CWD = os.getcwd()


NOW = datetime.now()
DATE = NOW.strftime("%y%m%d")

logging.basicConfig(
    filename=f"{CWD}\\__subsearch__.log",
    filemode="w",
    level=logging.DEBUG,
    format="%(asctime)s - %(message)s",
    datefmt="%d-%b-%y %H:%M:%S",
)

# log and print message
def output(msg: str, print_to_terminal: bool = True) -> None:
    logging.info(msg)
    return print(msg) if print_to_terminal else False


# log and print all the used parameters from video/directory-name
def parameters(param: str, language: str, lang_abbr: str, hearing_impaired: str, percentage: int) -> None:
    output(f"SubSearch - {__version__} ", False)
    output("[PARAMETERS]")
    output(f"Language: {language}, {lang_abbr}")
    output(f"Hearing impaired: {hearing_impaired}")
    output(f"Title: {param.title}")
    output(f"Year: {param.year}")
    output(f"Season: {param.season}, {param.season_ordinal}")
    output(f"Episode: {param.episode}, {param.episode_ordinal}")
    output(f"TV-Series: {param.tv_series}")
    output(f"Release: {param.release}")
    output(f"Group: {param.group}")
    output(f"Match threshold: {percentage}%")
    output(f"File hash: {param.file_hash}")
    output(f"URL: {param.url_subscene}")
    output(f"URL: {param.url_opensubtitles}\n")



from .__set_data__ import Paths

__root__ = getattr(Paths, "root")
__data__ = getattr(Paths, "data")
__gui__ = getattr(Paths, "gui")
__scraper__ = getattr(Paths, "scraper")
__utils__ = getattr(Paths, "utils")
__icons__ = getattr(Paths, "icons")
__buttons__ = getattr(Paths, "buttons")



from .__set_data__ import VideoFile

__video_name__ = getattr(VideoFile, "name")
__video_ext__ = getattr(VideoFile, "ext")
__video_path__ = getattr(VideoFile, "path")
__video_directory__ = getattr(VideoFile, "directory")



from dataclasses import dataclass

from num2words import num2words


@dataclass
class SearchParameters:
    url_subscene: str
    url_opensubtitles: str
    title: str
    year: int
    season: str
    season_ordinal: str
    episode: str
    episode_ordinal: str
    tv_series: bool
    release: str
    group: str
    file_hash: str


def split_last_hyphen(string: str) -> str:
    """
    Split last hyphen in a string, keep last part of the string

    Args:
        string (str): any string with a hyphen

    Returns:
        str: last item in the list of the string split by hyphen
    """
    group = string.rsplit("-", 1)[-1]
    return group


def get_parameters(file_name: str, file_hash: str, lang_abbr: str) -> SearchParameters:
    """
    Parse file name and get parameters for searching on subscene and opensubtitles

    Args:
        file_name (str): name of the file
        file_hash (str): hash of the file
        lang_abbr (str): language abbreviation for ordinal numbers

    Returns:
        SearchParameters: title, year, season, season_ordinal, episode, episode_ordinal, tv_series, release, group
    """
    # default values
    title = "N/A"
    year = "N/A"
    season = "N/A"
    season_ordinal = "N/A"
    episode = "N/A"
    episode_ordinal = "N/A"
    group = "N/A"
    tv_series = False
    year_found = False

    release = file_name

    # find season, episode, make it ordinal and set tv_series to true if it is a tv-series
    for item in release.lower().split("."):
        if item.startswith("s") and item[-1].isdigit() and "e" in item:
            season, episode = item.replace("s", "").replace("e", " ").split(" ")
            season_ordinal = num2words(int(season), lang=lang_abbr, to="ordinal")
            episode_ordinal = num2words(int(episode), lang=lang_abbr, to="ordinal")
            if season[-1].isdigit():
                tv_series = True
                break

    string_list = release.lower().split(".")
    subtract = []
    # list is reversed if a year is apart of the title, like "2001: A Foo Bar" from 1991
    # this is to prevent the year from the title being used as the release year of the movie/show
    for item in string_list[::-1]:
        # if year is found everything else in the list should be the release name
        if year_found or item.startswith("s") and item[-1].isdigit():
            _title = (i for i in string_list if i not in subtract)
            break
        if item.isdigit() and len(item) == 4:
            year_found = True
            year = item
        subtract.append(item)

    if tv_series:
        title = " ".join(x for x in _title).replace(f"s{season}e{episode}", f"- {season_ordinal} season").strip()
    else:
        title = " ".join(x for x in _title)

    if "-" in release:
        group = split_last_hyphen(release)

    subscene = "https://subscene.com/subtitles/searchbytitle?query="
    opensubtitles = "https://www.opensubtitles.org/en/search/sublanguageid-eng/moviename-"
    url_subscene = f"{subscene}{title}".replace(" ", "%20")
    url_opensubtitles = f"{opensubtitles}{file_hash}"

    parameters = {
        "url_subscene": url_subscene,
        "url_opensubtitles": url_opensubtitles,
        "title": title,
        "year": year,
        "season": season,
        "season_ordinal": season_ordinal,
        "episode": episode,
        "episode_ordinal": episode_ordinal,
        "tv_series": tv_series,
        "release": release,
        "group": group,
        "file_hash": file_hash,
    }
    return SearchParameters(**parameters)



import os
import sys
from itertools import product


class Paths:
    root: str
    data: str
    gui: str
    scraper: str
    utils: str
    icons: str
    buttons: str


class VideoFile:
    name: str
    ext: str
    path: str
    directory: str


class SetValues:
    def __init__(self) -> None:
        self._video()
        self._paths()

    def _paths(self) -> Paths:
        """
        Set all the paths SubSearch uses

        Returns:
            Paths: root, data, gui, scraper, utils, icons, buttons
        """
        root = os.path.abspath(os.path.join(os.path.dirname(__file__), "../"))
        all_paths = {
            "root": root,
            "data": os.path.join(root, "data"),
            "gui": os.path.join(root, "gui"),
            "scraper": os.path.join(root, "scraper"),
            "utils": os.path.join(root, "utils"),
            "icons": os.path.join(root, "assets", "icons"),
            "buttons": os.path.join(root, "assets", "buttons"),
        }
        for k, v in all_paths.items():
            setattr(Paths, k, v)

    def _video(self) -> VideoFile:
        """
        Set path, name, directory and ext for the video file

        Returns:
            VideoFilePaths: name, ext, path, directory
        """
        exts = [
            ".avi",
            ".mp4",
            ".mkv",
            ".mpg",
            ".mpeg",
            ".mov",
            ".rm",
            ".vob",
            ".wmv",
            ".flv",
            ".3gp",
            ".3g2",
            ".swf",
            ".mswmm",
        ]
        file_exist = False
        for i in product(exts, sys.argv):
            if i[1].endswith(i[0]) and str(i[1])[i[1].rfind("\\") :].startswith("\\"):
                path = i[1]
                directory = path[: path.rfind("\\")]
                name = path[path.rfind("\\") + 1 :].rsplit(".", 1)[0]
                ext = f".{name.rsplit('.', 1)[-1]}"
                file_exist = True
                break

        if file_exist:
            setattr(VideoFile, "name", name)
            setattr(VideoFile, "ext", ext)
            setattr(VideoFile, "path", path)
            setattr(VideoFile, "directory", directory)
        else:
            setattr(VideoFile, "path", None)
            setattr(VideoFile, "ext", None)
            setattr(VideoFile, "name", None)
            setattr(VideoFile, "directory", None)



import os
import shutil
import struct
import zipfile

import cloudscraper

from . import log, string_parser

SCRAPER = cloudscraper.create_scraper(browser={"browser": "chrome", "platform": "android", "desktop": False})


# download zip files from url
def download_zip(item: str) -> None:
    """
    download zip file from url

    Args:
        item (str): url to download
    """
    file_path, url, current_num, total_num = item
    log.output(f"Downloading: {current_num}/{total_num}")
    r = SCRAPER.get(url, stream=True)
    with open(file_path, "wb") as fd:
        for chunk in r.iter_content(chunk_size=1024):
            fd.write(chunk)


# extract all zip file in said directory
def extract_zips(cwd: str, extension: str) -> None:
    """
    extract all zip file in said directory that start with __subsearch__

    Args:
        cwd (str): directory to extract zip files from
        extension (str): extension to extract
    """
    subs_folder = os.path.join(cwd, "subs")
    if not os.path.exists(subs_folder):
        os.mkdir(subs_folder)
    for file in os.listdir(cwd):
        if file.startswith("__subsearch__") and file.endswith(extension):
            log.output(f"Extracting: {file} -> ..\\subs\\{file}")
            file_name = os.path.join(cwd, file)
            # file_name = os.path.abspath(file)
            zip_ref = zipfile.ZipFile(file_name)
            zip_ref.extractall(subs_folder)
            zip_ref.close()


# rename a .srts to the same as video release name
def rename_best_match(release_name: str, cwd: str, extension: str) -> None:
    """
    rename the best matching srt file compared to the video file for e.g MPC-HC to auto import

    Args:
        release_name (str): name of the video name
        cwd (str): current working directory
        extension (str): suffix of the subtitle file
    """
    higest_value = (0, "")
    subs_folder = os.path.join(cwd, "subs")
    for file in os.listdir(subs_folder):
        if file.endswith(extension):
            value = string_parser.pct_value(file, release_name)
            if value >= higest_value[0]:
                higest_value = value, file

    file_to_rename = higest_value[1]
    old_name_src = os.path.join(subs_folder, file_to_rename)
    new_name_dst = os.path.join(subs_folder, release_name)
    log.output(f"Renaming: {file} -> {release_name}")
    os.rename(old_name_src, new_name_dst)
    move_src = new_name_dst
    move_dst = os.path.join(cwd, release_name)
    log.output(f"Moving: {release_name} -> {cwd}")
    shutil.move(move_src, move_dst)


# remove .zips
def clean_up(cwd: str, extension: str) -> None:
    """
    Remove all the temporary files in the current working directory

    Args:
        cwd (str): current working directory
        extension (str): suffix of the file to remove
    """
    for file in os.listdir(cwd):
        if file.startswith("__subsearch__") and file.endswith(extension):
            log.output(f"Removing: {file}")
            file_path = os.path.join(cwd, file)
            os.remove(file_path)


# get file hash
def get_hash(file_name: str) -> str | None:
    """
    Tries to get the hash of the file

    Args:
        file_name (str): path/file_name to get the hash of

    Returns:
        str | None: the hash of the file or None if the size is 0
    """
    try:
        longlongformat = "<q"  # little-endian long long
        bytesize = struct.calcsize(longlongformat)
        with open(file_name, "rb") as f:
            filesize = os.path.getsize(file_name)
            hash = filesize
            if filesize < 65536 * 2:
                log.output(f"SizeError: filesize is {filesize} bytes", False)
                return None
            n1 = 65536 // bytesize
            for _x in range(n1):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF  # to remain as 64bit number
            f.seek(max(0, filesize - 65536), 0)
            n2 = 65536 // bytesize
            for _x in range(n2):
                buffer = f.read(bytesize)
                (l_value,) = struct.unpack(longlongformat, buffer)
                hash += l_value
                hash = hash & 0xFFFFFFFFFFFFFFFF

        returnedhash = "%016x" % hash
        return returnedhash

    except IOError as err:
        return log.output(err)



import json

from data import __data__


# update config.json
def set_json(key: str, value: str | int) -> None:
    config_file = f"{__data__}\\config.json"
    with open(config_file, "r+", encoding="utf-8") as f:
        data = json.load(f)
        data[key] = value
        f.seek(0)
        json.dump(data, f, indent=4)
        f.truncate()


def get_json() -> dict:
    config_file = f"{__data__}\\config.json"
    with open(config_file, encoding="utf-8") as file:
        data = json.load(file)

    return data


# get said value(s) from config.json
def get(output: str) -> str:
    config_json_dict = {
        "language": get_json()["language"].split(", "),
        "languages": get_json()["languages"],
        "other_languages": get_json()["other_languages"],
        "hearing_impaired": get_json()["hearing_impaired"],
        "percentage": get_json()["percentage_pass"],
        "cm_icon": get_json()["context_menu_icon"],
        "show_download_window": get_json()["show_download_window"],
        "show_terminal": get_json()["show_terminal"],
        "file_ext": get_json()["file_ext"],
    }
    return config_json_dict[f"{output}"]


# set default config.json values
def set_default_json() -> None:
    data = get_json()
    data["language"] = "English, en"
    data["hearing_impaired"] = "Both"
    data["percentage_pass"] = 90
    data["context_menu_icon"] = True
    data["show_download_window"] = True
    data["show_terminal"] = False
    data["file_ext"] = dict.fromkeys(data["file_ext"], True)
    config_file = f"{__data__}\\config.json"
    with open(config_file, "r+", encoding="utf-8") as file:
        file.seek(0)
        json.dump(data, file, indent=4)
        file.truncate()



# compare two lists  with each other and return match in %
def pct_value(from_pc: str | int, from_browser: str | int) -> int:
    """
    Parse two lists and return match in %

    Args:
        from_pc (str | int): list from pc
        from_browser (str | int): list from browser
    Returns:
        int: value in percentage of how many words match
    """
    max_percentage = 100
    pc_list: list = mk_lst(from_pc)
    browser_list: list = mk_lst(from_browser)
    not_matching = list(set(pc_list) - set(browser_list))
    not_matching_value = len(not_matching)
    number_of_items = len(pc_list)
    percentage_to_remove = round(not_matching_value / number_of_items * max_percentage)
    pct = round(max_percentage - percentage_to_remove)

    return pct


# create list from string
def mk_lst(release: str | int):
    new: list = []
    qualities = ["720p", "1080p", "1440p", "2160p"]
    temp: list = release.split(".")

    for item in temp:
        if item not in qualities:
            new.append(item.lower())
    return new



from .__set_data__ import SetValues

SetValues()
from .__paths__ import (
    __buttons__,
    __data__,
    __gui__,
    __icons__,
    __root__,
    __scraper__,
    __utils__,
)
from .__version__ import __version__
from .__video__ import (
    __video_directory__,
    __video_ext__,
    __video_name__,
    __video_path__,
)



import tkinter as tk

from data import __icons__

from . import tkinter_data as tkd
from . import tools


def main():
    global root
    root = tk.Tk(className=f" SubSearch")
    root.configure(background=tkd.Color.black)
    icon_path = f"{__icons__}\\16.ico"
    root.iconbitmap(icon_path)
    root.geometry(tools.WindowPosition.set(root))
    root.resizable(False, False)
    tools.CustomBorder(root).place(relx=0.5, rely=0.5, anchor="center")
    tk.Frame(root, height=40, bg=tkd.Color.dark_grey).pack(anchor="center", expand=True)
    tools.CustomTitleBar(root).place(x=tkd.Window.width - 2, y=2, bordermode="inside", anchor="ne")

    return root
